import pytest
import copy
import re

from typing import Callable

from metricflow.model.model_validator import ModelValidator
from metricflow.model.objects.data_source import DataSource, Mutability, MutabilityType
from metricflow.model.objects.elements.dimension import Dimension, DimensionType, DimensionTypeParams
from metricflow.model.objects.elements.identifier import IdentifierType, Identifier, CompositeSubIdentifier
from metricflow.model.objects.elements.measure import Measure, AggregationType
from metricflow.model.objects.metric import Metric, MetricType, MetricTypeParams
from metricflow.model.objects.user_configured_model import UserConfiguredModel
from metricflow.model.validations.validator_helpers import ModelValidationException
from metricflow.specs import DimensionReference, MeasureReference, IdentifierReference
from metricflow.time.time_granularity import TimeGranularity
from metricflow.test.test_utils import find_data_source_with


def test_data_source_cant_have_more_than_one_primary_identifier(
    simple_model__pre_transforms: UserConfiguredModel,
) -> None:  # noqa: D
    """Add an additional primary identifier to a data source and assert that it cannot have two"""
    model = copy.deepcopy(simple_model__pre_transforms)
    func: Callable[[DataSource], bool] = lambda data_source: len(data_source.identifiers) > 1

    multiple_identifier_data_source, _ = find_data_source_with(model, func)

    identifier_names = set()
    for identifier in multiple_identifier_data_source.identifiers:
        identifier.type = IdentifierType.PRIMARY
        identifier_names.add(identifier.name)

    build = ModelValidator.validate_model(model)

    future_issue = (
        f"Data sources can have only one primary identifier. The data source"
        f" `{multiple_identifier_data_source.name}` has {len(identifier_names)}"
    )

    found_future_issue = False

    if build.issues is not None:
        for issue in build.issues:
            if re.search(future_issue, issue.message):
                found_future_issue = True

    assert found_future_issue


def test_invalid_composite_identifiers() -> None:  # noqa:D
    with pytest.raises(ModelValidationException, match=r"If sub identifier has same name"):
        dim_reference = DimensionReference(element_name="time")
        measure_reference = MeasureReference(element_name="foo")
        measure2_reference = MeasureReference(element_name="metric_with_no_time_dim")
        identifier_reference = IdentifierReference(element_name="thorium")
        foreign_identifier_reference = IdentifierReference(element_name="composite_thorium")
        ModelValidator.checked_validations(
            UserConfiguredModel(
                data_sources=[
                    DataSource(
                        name="dim1",
                        sql_query=f"SELECT {dim_reference.element_name}, {measure_reference.element_name}, thorium_id FROM bar",
                        measures=[Measure(name=measure_reference, agg=AggregationType.SUM)],
                        dimensions=[
                            Dimension(
                                name=dim_reference,
                                type=DimensionType.TIME,
                                type_params=DimensionTypeParams(
                                    is_primary=True,
                                    time_format="YYYY-MM-DD",
                                    time_granularity=TimeGranularity.DAY,
                                ),
                            )
                        ],
                        identifiers=[
                            Identifier(name=identifier_reference, type=IdentifierType.PRIMARY, expr="thorium_id"),
                            Identifier(
                                name=foreign_identifier_reference,
                                type=IdentifierType.FOREIGN,
                                identifiers=[
                                    CompositeSubIdentifier(name=identifier_reference, expr="not_thorium_id"),
                                ],
                            ),
                        ],
                        mutability=Mutability(type=MutabilityType.IMMUTABLE),
                    ),
                ],
                metrics=[
                    Metric(
                        name=measure2_reference.element_name,
                        type=MetricType.MEASURE_PROXY,
                        type_params=MetricTypeParams(measures=[measure_reference]),
                    )
                ],
                materializations=[],
            )
        )


def test_composite_identifiers_nonexistent_ref() -> None:  # noqa:D
    with pytest.raises(ModelValidationException, match=r"Identifier ref must reference an existing identifier by name"):
        dim_reference = DimensionReference(element_name="time")
        measure_reference = MeasureReference(element_name="foo")
        measure2_reference = MeasureReference(element_name="metric_with_no_time_dim")
        identifier_reference = IdentifierReference(element_name="thorium")
        foreign_identifier_reference = IdentifierReference(element_name="composite_thorium")
        ModelValidator.checked_validations(
            UserConfiguredModel(
                data_sources=[
                    DataSource(
                        name="dim1",
                        sql_query=f"SELECT {dim_reference.element_name}, {measure_reference.element_name}, thorium_id FROM bar",
                        measures=[Measure(name=measure_reference, agg=AggregationType.SUM)],
                        dimensions=[
                            Dimension(
                                name=dim_reference,
                                type=DimensionType.TIME,
                                type_params=DimensionTypeParams(
                                    is_primary=True,
                                    time_format="YYYY-MM-DD",
                                    time_granularity=TimeGranularity.DAY,
                                ),
                            )
                        ],
                        identifiers=[
                            Identifier(name=identifier_reference, type=IdentifierType.PRIMARY, expr="thorium_id"),
                            Identifier(
                                name=foreign_identifier_reference,
                                type=IdentifierType.FOREIGN,
                                identifiers=[
                                    CompositeSubIdentifier(ref="ident_that_doesnt_exist"),
                                ],
                            ),
                        ],
                        mutability=Mutability(type=MutabilityType.IMMUTABLE),
                    ),
                ],
                metrics=[
                    Metric(
                        name=measure2_reference.element_name,
                        type=MetricType.MEASURE_PROXY,
                        type_params=MetricTypeParams(measures=[measure_reference]),
                    )
                ],
                materializations=[],
            )
        )


def test_composite_identifiers_ref_and_name() -> None:  # noqa:D
    with pytest.raises(ModelValidationException, match=r"Both ref and name/expr set in sub identifier of identifier"):
        dim_reference = DimensionReference(element_name="time")
        measure_reference = MeasureReference(element_name="foo")
        measure2_reference = MeasureReference(element_name="metric_with_no_time_dim")
        identifier_reference = IdentifierReference(element_name="thorium")
        foreign_identifier_reference = IdentifierReference(element_name="composite_thorium")
        foreign_identifier2_reference = IdentifierReference(element_name="shouldnt_have_both")
        ModelValidator.checked_validations(
            UserConfiguredModel(
                data_sources=[
                    DataSource(
                        name="dim1",
                        sql_query=f"SELECT {dim_reference.element_name}, {measure_reference.element_name}, thorium_id FROM bar",
                        measures=[Measure(name=measure_reference, agg=AggregationType.SUM)],
                        dimensions=[
                            Dimension(
                                name=dim_reference,
                                type=DimensionType.TIME,
                                type_params=DimensionTypeParams(
                                    is_primary=True,
                                    time_format="YYYY-MM-DD",
                                    time_granularity=TimeGranularity.DAY,
                                ),
                            )
                        ],
                        identifiers=[
                            Identifier(name=identifier_reference, type=IdentifierType.PRIMARY, expr="thorium_id"),
                            Identifier(
                                name=foreign_identifier_reference,
                                type=IdentifierType.FOREIGN,
                                identifiers=[
                                    CompositeSubIdentifier(
                                        ref="ident_that_doesnt_exist", name=foreign_identifier2_reference
                                    ),
                                ],
                            ),
                        ],
                        mutability=Mutability(type=MutabilityType.IMMUTABLE),
                    ),
                ],
                metrics=[
                    Metric(
                        name=measure2_reference.element_name,
                        type=MetricType.MEASURE_PROXY,
                        type_params=MetricTypeParams(measures=[measure_reference]),
                    )
                ],
                materializations=[],
            )
        )



import logging

import yaml
from jsonschema import exceptions

from metricflow.errors.errors import ParsingException
from metricflow.model.parsing.schemas_internal import (
    metric_validator,
    data_source_validator,
    materialization_validator,
)

VERSION_KEY = "version"
METRIC_TYPE = "metric"
DATA_SOURCE_TYPE = "data_source"
MATERIALIZATION_TYPE = "materialization"
DERIVED_GROUP_BY_ELEMENT_TYPE = "derived_identifier"
DOCUMENT_TYPES = [METRIC_TYPE, DATA_SOURCE_TYPE, MATERIALIZATION_TYPE]

logger = logging.getLogger(__name__)


def validate_config_structure(contents: str) -> None:  # noqa: D
    """Validates config shape against jsonschema

    catches ValidationError and raise one exception at the end so
    we can get all the validation errors rather than just the first
    """
    errors = []
    for config_document in yaml.load_all(contents, Loader=yaml.SafeLoader):
        # The config document can be None if there is nothing but white space between two `---`
        # this isn't really an issue, so lets just swallow it
        if config_document is None:
            continue
        if not isinstance(config_document, dict):
            errors.append(
                str(
                    ParsingException(
                        f"Transform object yaml must be a dict. Got `{type(config_document)}`: {config_document}",
                    )
                )
            )
            continue
        for document_type in config_document.keys():
            try:
                if document_type == METRIC_TYPE:
                    metric_validator.validate(config_document[document_type])
                elif document_type == DATA_SOURCE_TYPE:
                    data_source_validator.validate(config_document[document_type])
                elif document_type == MATERIALIZATION_TYPE:
                    materialization_validator.validate(config_document[document_type])
                elif document_type == DERIVED_GROUP_BY_ELEMENT_TYPE:
                    logger.warning(f"{DERIVED_GROUP_BY_ELEMENT_TYPE} is not supported")
                elif document_type == VERSION_KEY:
                    pass
                else:
                    raise ParsingException(f"No valid document type detected. got: {document_type}")
            except exceptions.ValidationError as e:
                errors.append(f"{e}")
                logger.exception(str(e))

    if len(errors) > 0:
        raise exceptions.ValidationError("\n".join(errors))



from typing import Dict

import yaml

"""This is the name of the field the SafeLineLoader adds to the parsed yaml
   we retrieve line number and the file name from the context stored at this key"""
PARSING_CONTEXT_KEY = "__parsing_context__"


class ParsingContext:  # noqa: D
    def __init__(self, start_line: int, end_line: int, filename: str) -> None:  # noqa: D
        self.start_line = start_line
        self.end_line = end_line
        self.filename = filename

    def __str__(self) -> str:  # noqa: D
        return f"line: {self.start_line}, filename: {self.filename}"


class SafeLineLoader(yaml.SafeLoader):
    """Adds special field __parsing_context__ to all mappings.

    Credit: https://stackoverflow.com/questions/13319067/parsing-yaml-return-with-line-number
    """

    # we may also want to consider this parser https://yaml.readthedocs.io/en/latest/
    # which supports yaml1.2 and maintains round-trip parsing, for now we use
    # the more established road
    # https://stackoverflow.com/questions/55441300/how-can-i-get-the-parent-node-within-yaml-loader-add-contructor
    def construct_mapping(self, node: yaml.MappingNode, deep: bool = False) -> Dict:  # noqa: D
        mapping = super(SafeLineLoader, self).construct_mapping(node, deep=deep)
        mapping[PARSING_CONTEXT_KEY] = ParsingContext(
            node.start_mark.line + 1, node.end_mark.line, node.start_mark.name
        )  # change to 1-indexed

        return mapping



from __future__ import annotations

import functools
import traceback
from abc import ABC, abstractmethod
from collections import OrderedDict
from dataclasses import dataclass
from datetime import date
from enum import Enum
from typing import Any, Callable, List, Optional, Tuple, Union

from metricflow.model.objects.elements.dimension import DimensionType
from metricflow.model.objects.user_configured_model import UserConfiguredModel

VALIDATE_SAFELY_ERROR_STR_TMPLT = ". Issue occurred in method `{method_name}` called with {arguments_str}"


class ValidationIssueLevel(Enum):
    """Categorize the issues found while validating a MQL model."""

    # Issue should be fixed, but model will still work in MQL
    WARNING = 0
    # Issue doesn't prevent model from working in MQL yet, but will eventually be an error
    FUTURE_ERROR = 1
    # Issue will prevent the model from working in MQL
    ERROR = 2
    # Issue is blocking and further validation should be stopped.
    FATAL = 3


class ModelObjectType(Enum):
    """Maps object types in the models to a readable string."""

    DATA_SOURCE = "data_source"
    MATERIALIZATION = "materialization"
    MEASURE = "measure"
    DIMENSION = "dimension"
    IDENTIFIER = "identifier"
    METRIC = "metric"


@dataclass(unsafe_hash=True)
class ValidationIssue:
    """An issue that was found while validating the MetricFlow model."""

    level: ValidationIssueLevel
    # Need a better way to represent the object, but right now this is a dict to the type of object to the name,
    # inserted in hierarchical order.
    # e.g. {"data_source": "example_data_source", "measure": "example_measure"}
    model_object_reference: OrderedDict[ModelObjectType, str]
    message: str
    # Consider adding a enum here that categories the type of validation issue and standardize the error messages.

    def as_readable_str(self) -> str:
        """Return a easily readable string that can be used to log the issue."""
        object_str = "{" + ", ".join([f"{k.value}: {v}" for k, v in self.model_object_reference.items()]) + "}"
        return f"[{self.level.name}] Object: {object_str} - {self.message}"

    @staticmethod
    def make_object_reference(
        data_source_name: Optional[str] = None,
        materialization_name: Optional[str] = None,
        dimension_name: Optional[str] = None,
        identifier_name: Optional[str] = None,
        measure_name: Optional[str] = None,
        metric_name: Optional[str] = None,
        object_type: Optional[ModelObjectType] = None,
        object_name: Optional[str] = None,
    ) -> OrderedDict[ModelObjectType, str]:
        """Convenience function for making the object reference dict."""

        model_object: OrderedDict[ModelObjectType, str] = OrderedDict()
        if data_source_name:
            if object_type == ModelObjectType.DATA_SOURCE:
                raise RuntimeError("Conflicting object types - multiple data sources")
            model_object[ModelObjectType.DATA_SOURCE] = data_source_name
        if materialization_name:
            if object_type == ModelObjectType.MATERIALIZATION:
                raise RuntimeError("Conflicting object types - multiple materializations")
            model_object[ModelObjectType.MATERIALIZATION] = materialization_name
        if metric_name:
            if object_type == ModelObjectType.METRIC:
                raise RuntimeError("Conflicting object types - multiple metrics")
            model_object[ModelObjectType.METRIC] = metric_name
        if dimension_name:
            if object_type == ModelObjectType.DIMENSION:
                raise RuntimeError("Conflicting object types - multiple dimensions")
            model_object[ModelObjectType.DIMENSION] = dimension_name
        if identifier_name:
            if object_type == ModelObjectType.IDENTIFIER:
                raise RuntimeError("Conflicting object types - multiple dimensions")
            model_object[ModelObjectType.IDENTIFIER] = identifier_name
        if measure_name:
            if object_type == ModelObjectType.MEASURE:
                raise RuntimeError("Conflicting object types - multiple dimensions")
            model_object[ModelObjectType.MEASURE] = measure_name

        if object_type:
            if not object_name:
                raise RuntimeError("Object name must be specified with object type.")
            model_object[object_type] = object_name

        if len(model_object) == 0:
            raise RuntimeError("No object parameters were passed in.")

        return model_object


@dataclass(unsafe_hash=True)
class ValidationWarning(ValidationIssue):
    """A warning that was found while validation the model."""

    def __init__(self, model_object_reference: OrderedDict[ModelObjectType, str], message: str):
        """Initializes with super (ValidationIssue) with hardcoded level of WARNING"""
        super().__init__(
            level=ValidationIssueLevel.WARNING, model_object_reference=model_object_reference, message=message
        )


@dataclass(unsafe_hash=True)
class ValidationFutureError(ValidationIssue):
    """A future error that was found while validation the model."""

    error_date: date

    def __init__(self, model_object_reference: OrderedDict[ModelObjectType, str], message: str, error_date: date):
        """Calls super (ValidationIssue) with hardcoded level of FUTURE_ERROR"""
        # Special way to set error_date because we're in a frozen dataclass
        object.__setattr__(self, "error_date", error_date)
        super().__init__(
            level=ValidationIssueLevel.FUTURE_ERROR,
            model_object_reference=model_object_reference,
            message=message,
        )

    def as_readable_str(self) -> str:
        """Return a easily readable string that can be used to log the issue."""
        object_str = "{" + ", ".join([f"{k.value}: {v}" for k, v in self.model_object_reference.items()]) + "}"
        return (
            f"[{self.level.name}] Object: {object_str} - {self.message}. "
            f"IMPORTANT: this error will break your model starting {self.error_date.strftime('%b %d, %Y')}. "
        )


@dataclass(unsafe_hash=True)
class ValidationError(ValidationIssue):
    """An error that was found while validating the model."""

    def __init__(self, model_object_reference: OrderedDict[ModelObjectType, str], message: str):
        """Calls super (ValidationIssue) with hardcoded level of ERROR"""
        super().__init__(
            level=ValidationIssueLevel.ERROR, model_object_reference=model_object_reference, message=message
        )


@dataclass(unsafe_hash=True)
class ValidationFatal(ValidationIssue):
    """A fatal issue that was found while validation the model."""

    def __init__(self, model_object_reference: OrderedDict[ModelObjectType, str], message: str):
        """Calls super (ValidationIssue) with hardcoded level of FATAL"""
        super().__init__(
            level=ValidationIssueLevel.FATAL, model_object_reference=model_object_reference, message=message
        )


ValidationIssueType = Union[ValidationIssue, ValidationWarning, ValidationFutureError, ValidationError, ValidationFatal]


def generate_exception_issue(
    obj_ref: OrderedDict[ModelObjectType, str],
    what_was_being_done: str,
    e: Exception,
) -> ValidationIssue:
    """Generates a validation issue for exceptions"""
    return ValidationError(
        model_object_reference=obj_ref,
        message=f"An error occured while {what_was_being_done} - {''.join(traceback.format_exception(etype=type(e), value=e, tb=e.__traceback__))}",
    )


def _func_args_to_string(*args: Any, **kwargs: Any) -> str:  # type: ignore
    return f"positional args: {args}, key word args: {kwargs}"


def validate_safely(whats_being_done: str) -> Callable:
    """Decorator to safely run validation checks"""

    def decorator_check_element_safely(func: Callable) -> Callable:  # noqa
        @functools.wraps(func)
        def wrapper(*args: Any, **kwargs: Any) -> List[ValidationIssueType]:  # type: ignore
            """Safely run a check on model elements"""
            issues: List[ValidationIssueType]
            try:
                issues = func(*args, **kwargs)
            except Exception as e:
                arguments_str = _func_args_to_string(*args, **kwargs)
                issues = [
                    generate_exception_issue(
                        obj_ref=OrderedDict(),
                        what_was_being_done=whats_being_done
                        + VALIDATE_SAFELY_ERROR_STR_TMPLT.format(
                            method_name=func.__name__, arguments_str=arguments_str
                        ),
                        e=e,
                    )
                ]
            return issues

        return wrapper

    return decorator_check_element_safely


@dataclass(frozen=True)
class DimensionInvariants:
    """Helper object to ensure consistent dimension attributes across data sources.

    All dimensions with a given name in all data sources should have attributes matching these values.
    """

    type: DimensionType
    is_partition: bool


class ModelValidationRule(ABC):
    """Encapsulates logic for checking the values of objects in a model."""

    @staticmethod
    @abstractmethod
    def validate_model(model: UserConfiguredModel) -> List[ValidationIssueType]:
        """Check the given model and return a list of validation issues"""
        pass


class ModelValidationException(Exception):
    """Exception raised when validation of a model fails."""

    def __init__(self, issues: Tuple[ValidationIssueType, ...]) -> None:  # noqa: D
        issues_str = "\n".join([x.as_readable_str() for x in issues])
        super().__init__(f"Error validating model. Issues:\n{issues_str}")



from __future__ import annotations

from typing import Optional

from metricflow.model.objects.utils import HashableBaseModel
from metricflow.object_utils import ExtendedEnum
from metricflow.specs import LinkableElementReference


class Version(HashableBaseModel):  # noqa: D
    major: int
    minor: int

    @staticmethod
    def parse(version: str) -> Version:  # noqa: D
        if version[0] == "v":
            version = version[1:]

        parts = version.split(".")
        if len(parts) != 2:
            raise RuntimeError(f"Invalid version string in configs: {version}")

        return Version(major=int(parts[0]), minor=int(parts[1]))


class Element:  # noqa: D
    name: LinkableElementReference
    expr: Optional[str]
    type: ExtendedEnum

    @property
    def is_primary_time(self) -> bool:  # noqa: D
        raise NotImplementedError(
            f"Subclasses of Element must implement `is_primary_time`. This object is of type {type(self)}"
        )


class SourceFile(HashableBaseModel):  # noqa: D
    path: str
    contents: str


class Commit(HashableBaseModel):  # noqa: D
    commit: str
    timestamp: int


class FileSlice(HashableBaseModel):  # noqa: D
    filename: str
    content: str
    start_line_number: int
    end_line_number: int


class YamlConfigFile(HashableBaseModel):  # noqa: D
    filepath: str
    contents: str
    url: str


class SourceContext(HashableBaseModel):  # noqa: D
    definition_hash: str



import copy
import logging
from typing import List

from metricflow.model.objects.user_configured_model import UserConfiguredModel
from metricflow.model.parsing.dir_to_model import ModelBuildResult
from metricflow.model.validations.common_identifiers import CommonIdentifiersRule
from metricflow.model.validations.data_sources import (
    DataSourceMeasuresUniqueRule,
    DataSourceTimeDimensionWarningsRule,
)
from metricflow.model.validations.dimension_const import DimensionConsistencyRule
from metricflow.model.validations.element_const import ElementConsistencyRule
from metricflow.model.validations.identifiers import IdentifierConfigRule, OnePrimaryIdentifierPerDataSourceRule
from metricflow.model.validations.metrics import MetricMeasuresRule, CumulativeMetricRule
from metricflow.model.validations.non_empty import NonEmptyRule
from metricflow.model.validations.unique_valid_name import UniqueAndValidNameRule
from metricflow.model.validations.validator_helpers import (
    ValidationIssueType,
    ModelValidationRule,
    ValidationIssueLevel,
    ModelValidationException,
)

logger = logging.getLogger(__name__)


class ModelValidator:
    """A Validator that acts on UserConfiguredModel"""

    VALIDATION_RULES: List[ModelValidationRule] = [
        CommonIdentifiersRule(),
        DataSourceMeasuresUniqueRule(),
        DataSourceTimeDimensionWarningsRule(),
        DimensionConsistencyRule(),
        ElementConsistencyRule(),
        IdentifierConfigRule(),
        OnePrimaryIdentifierPerDataSourceRule(),
        MetricMeasuresRule(),
        CumulativeMetricRule(),
        NonEmptyRule(),
        UniqueAndValidNameRule(),
    ]

    @staticmethod
    def validate_model(model: UserConfiguredModel) -> ModelBuildResult:
        """Validate a model according to configured rules."""
        model_copy = copy.deepcopy(model)

        issues: List[ValidationIssueType] = []
        for validation_rule in ModelValidator.VALIDATION_RULES:
            issues.extend(validation_rule.validate_model(model_copy))
            # If there are any fatal errors, stop the validation process.
            if any([x.level == ValidationIssueLevel.FATAL for x in issues]):
                return ModelBuildResult(model=model_copy, issues=tuple(issues))

        # If there are any errors, don't run any transforms and return the issues found.
        if any([x.level == ValidationIssueLevel.ERROR for x in issues]):
            return ModelBuildResult(model=model_copy, issues=tuple(issues))

        return ModelBuildResult(model=model_copy, issues=tuple(issues))

    @staticmethod
    def checked_validations(model: UserConfiguredModel) -> UserConfiguredModel:  # chTODO: remember checked_build
        """Similar to validate(), but throws an exception if validation fails."""
        model_copy = copy.deepcopy(model)
        build_result = ModelValidator.validate_model(model_copy)
        if build_result.issues is not None:
            if any(
                [
                    x.level == ValidationIssueLevel.WARNING or x.level == ValidationIssueLevel.FUTURE_ERROR
                    for x in build_result.issues
                ]
            ):
                issues_str = "\n".join([x.as_readable_str() for x in build_result.issues])
                logger.warning(f"Found some validation warnings in the model:\n{issues_str}")
            if any(
                [
                    x.level == ValidationIssueLevel.ERROR or x.level == ValidationIssueLevel.FATAL
                    for x in build_result.issues
                ]
            ):
                raise ModelValidationException(issues=build_result.issues)
        return model

    # chTODO: This is commented out in case during migration I learn I need it. If that happens, I'll move it
    # appropriately. Before the final migration of validations, this long comment will disappear from here.

    # def __init__(self, model: UserConfiguredModel, _called_from_build: bool = False):
    #     """Create a validated and transformed model from the given input model.
    #
    #     This shouldn't be called directly. Trying to make a private constructor by using _called_from_build.
    #     """
    #
    #     if not _called_from_build:
    #         raise RuntimeError("This should only be called from *build methods.")
    #     # Following sections depend on validations above being run, so no checks are included.
    #     # Stores the names of top level objects (like data sources, and metrics, that should be unique)
    #     self._top_level_object_names = set()
    #     if model.data_sources:
    #         for data_source in model.data_sources:
    #             self._top_level_object_names.add(data_source.name)
    #     if model.materializations:
    #         for materialization in model.materializations:
    #             self._top_level_object_names.add(materialization.name)
    #
    #     self._primary_time_dimension_name: DimensionSpec
    #     # Store dimension invariants
    #     self._dimension_invariants: Dict[DimensionReference, DimensionInvariants] = {}
    #     for data_source in model.data_sources:
    #         dimensions = data_source.dimensions
    #         for dimension in dimensions or []:
    #             self._dimension_invariants[dimension.name] = DimensionInvariants(
    #                 dimension.type, dimension.is_partition or False
    #             )
    #             if (
    #                 dimension.type == DimensionType.TIME
    #                 and dimension.type_params
    #                 and dimension.type_params.is_primary is True
    #             ):
    #                 self._primary_time_dimension_name = dimension.name
    #
    #     # Store the element types
    #     self._element_types: Dict[str, ModelObjectType] = {}
    #     for data_source in model.data_sources:
    #         if data_source.measures:
    #             for measure in data_source.measures:
    #                 self._element_types[measure.name.element_name] = ModelObjectType.MEASURE
    #         if data_source.dimensions:
    #             for dimension in data_source.dimensions:
    #                 self._element_types[dimension.name.element_name] = ModelObjectType.DIMENSION
    #         if data_source.identifiers:
    #             for identifier in data_source.identifiers:
    #                 self._element_types[identifier.name.element_name] = ModelObjectType.IDENTIFIER
    #
    #     super().__init__(
    #         data_sources=model.data_sources,
    #         materializations=model.materializations,
    #         metrics=model.metrics,
    #         derived_group_by_elements=model.derived_group_by_elements,
    #     )
